<?php
/*

Code for processing an uploaded image and returning a JSON encoded
command that can be used by EyesBot Driver.

This is free and unencumbered software released into the public domain.

Anyone is free to copy, modify, publish, use, compile, sell, or
distribute this software, either in source code form or as a compiled
binary, for any purpose, commercial or non-commercial, and by any
means.

In jurisdictions that recognize copyright laws, the author or authors
of this software dedicate any and all copyright interest in the
software to the public domain. We make this dedication for the benefit
of the public at large and to the detriment of our heirs and
successors. We intend this dedication to be an overt act of
relinquishment in perpetuity of all present and future rights to this
software under copyright law.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Written by John Keogh of Upsoni, LLC (john.keogh@upsoni.com)

 */
 
 
//these are example setting for detecting an orange colored object, like a DFRobotics box
$RedUpperLimit = 255;
$RedLowerLimit = 230;
$GreenUpperLimit = 200;
$GreenLowerLimit = 100;
$BlueUpperLimit = 100;
$BlueLowerLimit = 0;

$standStillAboveRatio = 0.4; 
$standStillBelowRatio = 0.04;

$moveForwardBelowRatio = 0.1;
$moveBackwardAboveRatio = 0.2;
$stop = "false";//send anything other than false or nil to stop

$debug=false;

//leftvelocity and rightvelocity are ints from -10 to 10 which govern the speed of the 
//left and right wheels
$leftvelocity = "0";
$rightvelocity = "0";
//light is an int from 0 to 10.  Currently, anything over 0 is full on
$light = "0";
//duration, an int, is the number of milliseconds that this command is good for.  So if you 
//want the robot to go forward for 3 seconds, set this to 3000 and left and right each to 10.  
//If another command is received prior to this time being up, the new command replaces the 
//older command.  A hard lower limit of 100 ms is enforced in the iOS code
$duration = "2000"; 
//delay, an int  is the amount of milliseconds to skip until the next image is sent, a hard 
//lower limit of 500 ms is enforced in the iOS code that calls the web service 
$delay = "2000";
//send errors or addition information back.  Now only displayed on the iOS device streaming
//interface, not on the EyesBot Driver web interface
$errors = "";

//The file submitted by the call to the web service is a multipart form encoded 
//jpeg with a name of "userfile"

if(isset($_FILES["userfile"])){
   //reality check
   if (($_FILES["userfile"]["type"] != "image/jpg")&&($_FILES["userfile"]["type"] != "image/jpeg")){
      $errors.="Please use a jpeg format image";
   }
   else if ($_FILES["userfile"]["size"] > 1000000) {
      $errors.="The file was larger than 1MB";
   }
   else if ($_FILES["userfile"]["error"] > 0) {
      $errors.="There was a problem uploading the image: " . $_FILES["userfile"]["error"];
   }
   else {
      $im = imagecreatefromjpeg($_FILES["userfile"]["tmp_name"]);
      list($width, $height) = getimagesize($_FILES["userfile"]["tmp_name"]);
      
      $totalPoints = 0;
      $matchPoints = 0;
      $leftThirdMatchSpots = 1;
      $middleThirdMatchSpots = 1;
      $rightThirdMatchSpots = 1;
      
      $leftThirdBoundary = ($width/3.0);
      $middleThirdBoundary = ((2.0*$width)/3.0);

      //the robot can only go left, right or straight, so only need to determine whether the 
      //object to follow is ahead, to the left or to the right, not whether it is up or down
      for($column = 0; $column < $width; $column++){
         for($row = 0; $row < $height; $row++){
            $rgb = imagecolorat($im, $column, $row);
            $r = ($rgb >> 16) & 0xFF;
            $g = ($rgb >> 8) & 0xFF;
            $b = $rgb & 0xFF;

            if((($r>$RedLowerLimit)&&($r<=$RedUpperLimit))&&
               (($g>$GreenLowerLimit)&&($g<=$GreenUpperLimit))&&
               (($b>$BlueLowerLimit)&&($b<=$BlueUpperLimit))){

                if($column>$middleThirdBoundary){
                    $rightThirdMatchSpots++;
                }
                else if($column>$leftThirdBoundary){
                    $middleThirdMatchSpots++;
                }
                else{
                    $leftThirdMatchSpots++;
                }
                $matchPoints++;
            }
            $totalPoints++;
         }
      }

      //ratios
      $matchRatio = ($matchPoints)/$totalPoints;
      $leftRatio = ($leftThirdMatchSpots)/$totalPoints;
      $rightRatio = ($rightThirdMatchSpots)/$totalPoints;
      $centerRatio = ($middleThirdMatchSpots)/$totalPoints;
    
      //are there two many absolute Match spots
      if($matchRatio>$standStillAboveRatio){
         $leftvelocity = 0;
         $rightvelocity = 0;
      }//or too few, ie no target
      else if($matchRatio<$standStillBelowRatio){
         $leftvelocity = 0;
         $rightvelocity = 0;
      }
      //what third has the most Match spots?
      else if(($leftRatio>$rightRatio)&&($leftRatio>$centerRatio)){//left
         if($leftRatio<$moveForwardBelowRatio){
            $leftvelocity = 2;
            $rightvelocity = 8;
         }
         else if($leftRatio>$moveBackwardAboveRatio){
            $leftvelocity = -8;
            $rightvelocity = -2;
         }
      }
      else if(($rightRatio>$leftRatio)&&($rightRatio>$centerRatio)){//right
         if($rightRatio<$moveForwardBelowRatio){
            $leftvelocity = 8;
            $rightvelocity = 2;
         }
         else if($rightRatio>$moveBackwardAboveRatio){
            $leftvelocity = -2;
            $rightvelocity = -8;
         }
      }
      else{//center
         if($centerRatio<$moveForwardBelowRatio){
            $leftvelocity = 8;
            $rightvelocity = 8;
         }
         else if($centerRatio>$moveBackwardAboveRatio){
            $leftvelocity = -8;
            $rightvelocity = -8;
         }
      }
   }
}
else{
   $errors.="No file uploaded";
}

if($debug){
   echo "totalPoints: $totalPoints\r\n";
   echo "MatchPoints: $matchPoints\r\n";
   echo "leftThirdMatchSpots: $leftThirdMatchSpots\r\n";
   echo "middleThirdMatchSpots: $middleThirdMatchSpots\r\n";
   echo "rightThirdMatchSpots: $rightThirdMatchSpots\r\n";
   echo "leftThirdBoundary: $leftThirdBoundary\r\n";
   echo "middleThirdBoundary: $middleThirdBoundary\r\n";
   echo "matchRatio: $matchRatio\r\n";
   echo "leftRatio: $leftRatio\r\n";
   echo "rightRatio: $rightRatio\r\n";
   echo "centerRatio: $centerRatio\r\n";
}
else{
   echo "{\r\n";
   echo " \"actions\":[\r\n";
   echo "  {";
   echo "   \"duration\": $duration,\r\n";
   echo "   \"delay\": $delay,\r\n";
   echo "   \"left\": $leftvelocity,\r\n";
   echo "   \"right\": $rightvelocity,\r\n";
   echo "   \"light\": $light,\r\n";
   echo "   \"stop\": \"$stop\",\r\n";
   echo "   \"comment\": \"$errors\"\r\n";
   echo "  }\r\n";
   echo " ]\r\n";
   echo "}\r\n";
}

?>